## 题目描述

You are given a rooted tree with root in vertex $1$ . Each vertex is coloured in some colour.

Let's call colour $c$ dominating in the subtree of vertex $v$ if there are no other colours that appear in the subtree of vertex $v$ more times than colour $c$ . So it's possible that two or more colours will be dominating in the subtree of some vertex.

The subtree of vertex $v$ is the vertex $v$ and all other vertices that contains vertex $v$ in each path to the root.

For each vertex $v$ find the sum of all dominating colours in the subtree of vertex $v$ .

## 输入格式

The first line contains integer $n ( 1<=n<=10^{5} )$ — the number of vertices in the tree.

The second line contains $n$ integers $c_{i}$$ ( 1<=c_{i}<=n)$, $c_{i}$— the colour of the $i$ -th vertex.

Each of the next $n-1$ lines contains two integers $x_{j}$,$y_{j}$$ ( 1<=x_{j},y_{j}<=n)$ — the edge of the tree. The first vertex is the root of the tree.

## 输出格式

Print $n$ integers — the sums of dominating colours for each vertex.

## 题意翻译

- 有一棵 $n$ 个结点的以 $1$ 号结点为根的**有根树**。
- 每个结点都有一个颜色，颜色是以编号表示的， $i$ 号结点的颜色编号为 $c_i$。
- 如果一种颜色在以 $x$ 为根的子树内出现次数最多，称其在以 $x$ 为根的子树中占**主导地位**。显然，同一子树中可能有多种颜色占主导地位。
- 你的任务是对于每一个 $i\in[1,n]$，求出以 $i$ 为根的子树中，占主导地位的颜色的编号和。
- $n\le 10^5$, $c_i\le n$
- 输入输出样例

**输入 #1**

```
4
1 2 3 4
1 2
2 3
2 4
```

**输出 #1**

```
10 9 3 4
```

**输入 #2**

```
15
1 2 3 1 2 3 3 1 1 3 2 2 1 2 3
1 2
1 3
1 4
1 14
1 15
2 5
2 6
2 7
3 8
3 9
3 10
4 11
4 12
4 13
```

**输出 #2**

```
6 5 4 3 2 3 3 1 1 3 2 2 1 2 3
```

```cpp
#include<iostream>
#include<string>
#include<queue>
#include<stack>
#include<vector>
#include<map>
#include<cstdio>
#include<cstdlib>
#include<algorithm>
#include<set>
#include<list>
#include<iomanip>
#include<cstring>
#include<cmath>
#include<limits>
#include<cctype>
#include<unordered_map>
#include<complex>
#include<assert.h>
#pragma warning(disable:4996)
using namespace std;

#define debug(i) cerr<<"<debug> "<<#i<<" "<<i<<" <\\debug>"<<endl
#define mfor(i,a,b) for(int i=(a);i<=(b);i++)
#define mrep(i,a,b) for(int i=(a);i>=(b);i--)
#define lll __int128
#define Re register
#define mem(a,b) memset(a,(b),sizeof(a))
#define Design ios::sync_with_stdio(0),cin.tie(0),cout.tie(0)
#define endl '\n'
#define toang(x) ((x)*180.0/pi)	//转角度
#define torad(x) ((x)*pi/180.0)	//转弧度
typedef long long ll;
typedef unsigned long long ull;
typedef pair<int, int> intpair;
typedef pair<ll, ll> llpair;
typedef long double ld;
typedef double dl;
const ld pi = acos(-1.0);
const ld eps = 1e-5;
const int INF = 0x3f3f3f3f;
const ll INFLL = 0x3f3f3f3f3f3f3f3f;

const int maxn = 200010;
int sz[maxn];
int c[maxn];
int hvs[maxn];
ll cnt[maxn];
ll ans[maxn];
ll res;
set<int>e[maxn];
int nows;
ll nowh;

int predfs(int x, int fa)
{//预处理出每个节点的重儿子
	sz[x] = 1;
	for (auto it : e[x])
	{
		if (it == fa) continue;
		sz[x] += predfs(it, x);
		if (!hvs[x] || sz[it] > sz[hvs[x]]) hvs[x] = it;
	}
	return sz[x];
}

void add(int x, int fa, int val) 
{//计算结果或消除子树对之后答案的影响（val=-1）
	cnt[c[x]] += val;
	if (cnt[c[x]] > nowh) nowh = cnt[c[x]], res = c[x];
	else if (cnt[c[x]] == nowh) res += c[x];
	for (auto it : e[x])
	{
		if (it == fa || it == nows) continue;
		add(it, x, val);
	}
}

void dfs(int x, int fa, int kep) 
{
    //先对每个轻儿子进行一次不保留的遍历
	for (auto it : e[x])
	{
		if (it == fa || it == hvs[x]) continue;
		dfs(it, x, 0);
	}
    //单独遍历重儿子并保留结果
	if (hvs[x]) dfs(hvs[x], x, 1), nows = hvs[x];
	//计算结果
    add(x, fa, 1);
	nows = 0;
	ans[x] = res;
	if (!kep)
	{
        //清空答案
		add(x, fa, -1);
		res = 0;
		nowh = 0;
	}
}

int main()
{
	Design;
	int n;
	cin >> n;
	mfor(i, 1, n) cin >> c[i];
	mfor(i, 2, n)
	{
		int u, v;
		cin >> u >> v;
		e[u].emplace(v);
		e[v].emplace(u);
	}
	predfs(1, -1);
	dfs(1, -1, 1);
	mfor(i, 1, n) cout << ans[i] << " ";
}
```

