**常用于解决组合问题**

$\displaystyle g(x)=a_0+a_1x+a_2x^2+a_3x^3$

称 $g(x)$ 是序列 $a_0,a_1,a_2,a_3,....,$ 的生成函数。

$g(x)=1+x^n+x^{2n}+x^{3n}+....=\frac{1}{1-x^n}$  （无限项）

$g(x)=1+x^n+x^{2n}+x^{3n}+....+x^{mn}=\frac{1-x^{(m+1)n}}{1-x^{n}}$

虽然 $x$ 没有什么实际用处，但是 $x\in(-1,1)$

对于生成函数

$g(x)=\frac{1}{(1-x)^k}$ 第 $n$ 项系数为 $C_{n+k-1}^{k-1}$ 

## 例题 1

有 $1,2,3,4$ 克砝码各一枚，能称出几种重量，每种重量有几种方案？

$n$ 克砝码可以看成 $1+x^n$，$1$ 表示不取这个砝码，$x^n$ 表示取 $1$ 个

可以列出生成函数：

$g(x)=(1+x^1)(1+x^2)(1+x^3)(1+x^4)$

拆开括号得到：

$g(x)=1+x+x^2+2x^3+2x^4+2x^5+2x^6+2x^7+x^8+x^9+x^{10}$

由此可得：

```cpp
/*重量：*/	0	1	2	3	4	5	6	7	8	9	10
/*方案数*/ 1   1   1   2   2   2   2   2   1   1    1
```

即指数表示重量，系数表示方案数

## 例题 2

有 $1,2,3$ 分邮票各无限张，求某个数值的方案数

$n$ 分邮票可以看成 $(1+x^n+x^{2n}+x^{3n}+....)$

生成函数：

$g(x)=(1+x^1+x^2+x^3+....)(1+x^2+x^4+x^6+....)(1+x^3+x^6+x^9+....)$

拆开括号：

$g(x)=1+x^1+2x^2+2x^3+4x^4+....$

组合出 $4$ 的方案为 $[4,0,0],[2,1,0],[0,2,0],[1,0,1]$ ，对应 $x^4+(x^2×x^2)+x^4+(x^1×x^3)=x^4$

共 $4$ 种，即拆开括号后 $x^4$ 这一项的系数

## 公式

$g(x)=1+x^n+x^{2n}+x^{3n}+....=\frac{1}{1-x^n}$  （无限项）

$g(x)=1+x^n+x^{2n}+x^{3n}+....+x^{mn}=\frac{1-x^{(m+1)n}}{1-x^{n}}$

## 特殊情况

当序列（第一题序列为 $[1,1,1,2,2,2,2,....]$，第二题为 $[1,1,1,2,2,4,....]$）

为 $[1,1,1,1,1,...]$ 时

$g(x)=1+x^1+x^2+....=\frac{1}{1-x}$

$g^n(x)=\frac{1}{(1-x)^k}=\sum_{i}^{\infty} C_{i+k-1}^{k-1}x^i$

即 $g^n(x)$ 第 $i$ 项的系数为 $C_{i+k-1}^{k-1}$

## 常用场景

用于解决有限或无限物体的组合方案数问题

## 例题

#### [HDU1028](http://acm.hdu.edu.cn/showproblem.php?pid=1028)  [HDU1085](http://acm.hdu.edu.cn/showproblem.php?pid=1085) 

#### [洛谷P2000](https://www.luogu.org/problemnew/show/P2000)  数据范围丧心病狂，不推荐

## P2000

### 题目背景

公元 2000 年，根据预言家诺查丹玛斯的预言，世界就要毁灭了！！！

### 题目描述

为了拯救世界，小 a 和 uim 决定召唤出 kkksc03 大神和 lzn 大神。根据古籍记载，召唤出任何一位大神，都需要使用金木水火土五种五行神石来摆一个特定的大阵。而在古籍中，记载是这样的：

kkksc03 大神召唤方法：

金神石的块数必须是 66 的倍数。

木神石最多用 99 块。

水神石最多用 55 块。

火神石的块数必须是 44 的倍数。

土神石最多用 77 块。

lzn 大神召唤方法:

金神石的块数必须是 22 的倍数。

木神石最多用 11 块。

水神石的块数必须是 88 的倍数。

火神石的块数必须是 1010 的倍数。

土神石最多用 33 块。

现在是公元 1999 年 12 月 31 日，小 a 和 uim 从 00:00:00 开始找，一直找到 23:00:00，终于，还是没找到神石。不过，他们在回到家后在自家地窖里发现了一些奇怪的东西，一查古籍，哎呦妈呀，怎么不早点来呢？这里有一些混沌之石，可以通过敲击而衰变成五行神石。于是，他们拼命地敲，终于敲出了n块神石，在 23:59:59 完成了两座大阵。然而，kkksc03 大神和 lzn 大神确实出现了，但是由于能量不够，无法发挥神力。只有把所有用 n*n* 块神石可能摆出的大阵都摆出来，才能给他们充满能量。这下小 a 和 uim 傻了眼了，赶快联系上了你，让你帮忙算一下，一共有多少种大阵。

### 输入格式

输入一个正整数 n*n*。

### 输出格式

输出用 n*n* 块混沌之石能摆出的大阵的种数。

### 输入输出样例

**输入 #1**

```
2
```

**输出 #1**

```
15
```

## 说明/提示

数据范围：

$10^{100000}\leq n\lt 10^{100001}$

### 提示

由于现在已经是 23:59:59，所以你只有 0.5\texttt s0.5s 时间。（小 a 与 uim 需要 0.5\texttt s0.5s 排出所有阵法）

```cpp
#include<iostream>
#include<string>
#include<queue>
#include<stack>
#include<vector>
#include<map>
#include<cstdio>
#include<cstdlib>
#include<algorithm>
#include<set>
#include<list>
#include<iomanip>
#include<cstring>
#include<cmath>
#include<limits>
#include<cctype>
#include<unordered_map>
#include<complex>
#pragma warning(disable:4996)
using namespace std;

#define debug(i) cout<<"<debug> "<<#i<<" "<<i<<" <\\debug>"<<endl
#define mfor(i,a,b) for(int i=(a);i<=(b);i++)
#define mrep(i,a,b) for(int i=(a);i>=(b);i--)
#define lll __int128
#define Re register
#define mem(a,b) memset(a,(b),sizeof(a))
#define Design ios::sync_with_stdio(0),cin.tie(0),cout.tie(0)
#define endl '\n'
#define toang(x) ((x)*180.0/pi)	//转角度
#define torad(x) ((x)*pi/180.0)	//转弧度
typedef long long ll;
typedef unsigned long long ull;
typedef pair<int, int> intpair;
typedef pair<ll, ll> llpair;
typedef long double ld;
const ld pi = acos(-1);
const ld eps = 1e-7;
const int INF = 0x3f3f3f3f;
const ll INFLL = 0x3f3f3f3f3f3f3f3f;


const int maxn = 1000010;

const ll mod = 1004535809;
const ll G = 3;
//G为mod的原根
//G为3时常见的mod：998244353,1004535809,469762049
ll rev[maxn];

long long fpow(long long a, long long n, long long mod)
{
	if (n == 0) return 1;
	else if (n & 1) return a * fpow(a, n - 1, mod) % mod;
	else
	{
		long long num = fpow(a, n / 2, mod) % mod;
		return num * num % mod;
	}
}

void NTT(ll * s, int f, int n)
{
	for (ll i = 0; i < n; i++)
	{
		if (i < rev[i]) swap(s[i], s[rev[i]]);
	}
	for (ll i = 1; i < n; i <<= 1)
	{
		ll wn = fpow(G, (mod - 1) / (i << 1), mod);
		if (!~f) wn = fpow(wn, mod - 2, mod);
		ll p = i << 1;
		for (ll j = 0; j < n; j += p)
		{
			ll w = 1;
			for (ll k = 0; k < i; k++)
			{
				ll x = s[j + k];
				ll y = w * s[j + k + i];
				s[j + k] = x + y;
				s[j + k] %= mod;
				s[j + k + i] = x - y;
				s[j + k + i] %= mod;
				(s[j + k + i] += mod) %= mod;
				w *= wn;
				w %= mod;
			}
		}
	}
}

void getNTT(ll * a, ll * b, int n, int m)
{
	int l = 0;
	m += n;
	for (n = 1; n <= m; n <<= 1) l++;
	for (ll i = 0; i < n; i++) rev[i] = (rev[i >> 1] >> 1) | ((i & 1) << (l - 1));
	NTT(a, 1, n);
	NTT(b, 1, n);
	for (ll i = 0; i <= n; i++) a[i] = a[i] * b[i], a[i] %= mod;
	NTT(a, -1, n);
	ll inv = fpow(n, mod - 2, mod);
	for (int i = 0; i <= m; i++) a[i] *= inv, a[i] %= mod;
}

ll sa[maxn];
ll sb[maxn];

int main()
{
	//Design;
	string s;
	cin >> s;
	reverse(s.begin(), s.end());
	mfor(i, 0, (int)s.length() - 1)
	{
		sa[i] = s[i] - ll('0');
		sb[i] = s[i] - ll('0');
	}
	sa[0]++;
	mfor(i, 0, 100010)
	{
		if (sa[i] >= 10) sa[i + 1] += sa[i] / 10, sa[i] %= 10;
	}
	sb[0] += 2;
	mfor(i, 0, 100010)
	{
		if (sb[i] >= 10) sb[i + 1] += sb[i] / 10, sb[i] %= 10;
	}
	getNTT(sa, sb, 200010, 100010);
	mfor(i, 0, 300010)
	{
		if (sa[i] >= 10) sa[i + 1] += sa[i] / 10, sa[i] %= 10;
	}
	mem(sb, 0);
	mfor(i, 0, s.length() - 1) sb[i] = s[i] - ll('0');
	sb[0] += 3;
	mfor(i, 0, 100010)
	{
		if (sb[i] >= 10) sb[i + 1] += sb[i] / 10, sb[i] %= 10;
	}
	getNTT(sa, sb, 300010, 100010);
	mfor(i, 0, 400010)
	{
		if (sa[i] >= 10) sa[i + 1] += sa[i] / 10, sa[i] %= 10;
	}
	mem(sb, 0);
	mfor(i, 0, s.length() - 1) sb[i] = s[i] - ll('0');
	sb[0] += 4;
	mfor(i, 0, 100010)
	{
		if (sb[i] >= 10) sb[i + 1] += sb[i] / 10, sb[i] %= 10;
	}
	getNTT(sa, sb, 400010, 100010);
	mfor(i, 0, 500010)
	{
		if (sa[i] >= 10)
		{
			sa[i + 1] += sa[i] / 10;
			sa[i] %= 10;
		}
	}
	ll now = 0;
	mrep(i, 500010, 0)
	{
		now = now % 24 * 10 + sa[i];
		sa[i] = now / 24;
	}
	mfor(i, 0, 500010)
	{
		if (sa[i] >= 10)
		{
			sa[i + 1] += sa[i] / 10;
			sa[i] %= 10;
		}
	}
	bool f = false;
	mrep(i, 500010, 0)
	{
		if (sa[i]) f = true;
		if (f) putchar(sa[i] + '0');//cout << sa[i];
	}
}
```

